package Interface;

import java.awt.BasicStroke;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.Iterator;

import Main.Edge;
import Main.Node;
import Utils.Angle;
import Utils.Vector2;

public class GridLayer extends Layer {
	
	private ArrayList<Vector2> _edges = new ArrayList<Vector2>();
	private ArrayList<Vector2> _dedges = new ArrayList<Vector2>();
	private Vector2 _orig;
	private Vector2 _dir1;
	private Vector2 _dir2;
	protected int _grid_radius;
	private ArrayList<Node> hexaList = new ArrayList<Node>();
	private ArrayList<Edge> edgeList = new ArrayList<Edge>();;

	protected GridLayerEmbedding _embedding;

	private GridLayerInterviewer _m;

	public GridLayer(JTransformPanel p, GridLayerEmbedding e, GridLayerInterviewer m) {
		super(p);
		_m = m;
		_embedding = e;
		on_transformation();
	}

	final public void compute_dir_vectors() {
		_orig = gridToPixel(0, 0);
		_dir1 = gridToPixel(1, 0);
		_dir2 = gridToPixel(0, 1);
		_dir1.untranslate(_orig);
		_dir2.untranslate(_orig);

		// System.out.println(_orig);
	}

	public void compute_grid_radius() {
		Vector2 p = new Vector2(_p.get_unzoom(_p.get_w_2()), _p.get_unzoom(_p.get_h_2()));
		_grid_radius = (int) Math.ceil(Math.sqrt(2) * p.norm() / _embedding.get_node_dist());
		// System.out.println(_grid_radius);
	}

	@Override
	public void on_transformation() {
		compute_dir_vectors();
		compute_grid_radius();
	}

	public void add_edge(int X, int Y) {
		// assert((X!=0)&&(Y!=0));
		_edges.add(new Vector2(X, Y));
		compute_dedges();
	}

	private void compute_dedges() {
		int i, N = _edges.size();
		Vector2 v1, v2;
		_dedges = new ArrayList<Vector2>();

		for (i = 0; i < N - 1; i++) {
			v1 = _edges.get(i);
			v2 = _edges.get(i + 1);
			_dedges.add(new Vector2(v2.x - v1.x, v2.y - v1.y));
		}

		v1 = _edges.get(N - 1);
		v2 = _edges.get(0);
		_dedges.add(new Vector2(-v2.x - v1.x, -v2.y - v1.y));

		for (i = 0; i < N - 1; i++) {
			v1 = _edges.get(i);
			v2 = _edges.get(i + 1);
			_dedges.add(new Vector2(v1.x - v2.x, v1.y - v2.y));
		}

		v1 = _edges.get(N - 1);
		v2 = _edges.get(0);
		_dedges.add(new Vector2(v2.x + v1.x, v2.y + v1.y));

		/*
		 * System.out.println("Edge basis:"); Iterator<Vector> edge =
		 * _edges.iterator(); while(edge.hasNext()) { Vector e = edge.next();
		 * System.out.println("  - "+e.x+" x "+e.y); }
		 * System.out.println("DEdge basis:"); Iterator<Vector> dedge =
		 * _dedges.iterator(); while(dedge.hasNext()) { Vector e = dedge.next();
		 * System.out.println("  - "+e.x+" x "+e.y); }
		 */
	}

	public void gridToLocal(Vector2 p) {
		_embedding.gridToLocal(p);
	}

	public void localToGrid(Vector2 p) {
		_embedding.localToGrid(p);
	}

	public void gridToGlobal(Vector2 p) {
		gridToLocal(p);
		_p.localToGlobal(p);
	}

	public void gridToPixel(Vector2 p) {
		gridToLocal(p);
		_p.localToPixel(p);
	}

	public void globalToGrid(Vector2 p) {
		_p.globalToLocal(p);
		localToGrid(p);
	}

	public void pixelToGrid(Vector2 p) {
		_p.pixelToGlobal(p);
		globalToGrid(p);
	}

	public Vector2 gridToLocal(int X, int Y) {
		Vector2 p = new Vector2((float) X, (float) Y);
		gridToLocal(p);
		return p;
	}

	public Vector2 localToGrid(float x, float y) {
		Vector2 p = new Vector2(x, y);
		localToGrid(p);
		return p;
	}

	public Vector2 gridToGlobal(int X, int Y) {
		Vector2 p = new Vector2((float) X, (float) Y);
		gridToGlobal(p);
		return p;
	}

	public Vector2 globalToGrid(float i, float j) {
		Vector2 p = new Vector2(i, j);
		globalToGrid(p);
		return p;
	}

	public Vector2 gridToPixel(int X, int Y) {
		Vector2 p = new Vector2((float) X, (float) Y);
		gridToPixel(p);
		return p;
	}

	public Vector2 pixelToGrid(int pi, int pj) {
		Vector2 p = new Vector2((float) pi, (float) pj);
		pixelToGrid(p);
		return p;
	}

	public void drawNode(Graphics g, int X, int Y) {
		if (!_m.node_to_draw(X, Y))
			return;

		String label = _m.get_node_label(X, Y);
		int pitch = _m.get_node_pitch(X,Y);
		
		float pi = _orig.x + X * _dir1.x + Y * _dir2.x;
		float pj = _orig.y + X * _dir1.y + Y * _dir2.y;
		if (g != null) {
			g.setColor(_m.get_node_color(X, Y));
			g.fillOval((int) (pi - _p.get_zoom(25.f)), (int) (pj - _p.get_zoom(25.f)), (int) (_p.get_zoom(50.f)),
					(int) (_p.get_zoom(50.f)));

			// g.setColor(_m.get_edge_color());
			g.setColor(_m.get_circle_color(X, Y));
			g.drawOval((int) (pi - _p.get_zoom(25.f)), (int) (pj - _p.get_zoom(25.f)), (int) (_p.get_zoom(50.f)),
					(int) (_p.get_zoom(50.f)));

			if (label != "") {
				float sx = 0.5f * g.getFontMetrics().stringWidth(label);
				float sy = 0.25f * g.getFontMetrics().getHeight();
				g.setColor(_m.get_label_color());
				g.drawString(label, (int) (pi - sx), (int) (pj + sy));
				// System.out.println(label + kk++);

			}
		}

		Ellipse2D.Double ellipse = new Ellipse2D.Double((pi - _p.get_zoom(25.f)), (pj - _p.get_zoom(25.f)),
				(_p.get_zoom(50.f)), (_p.get_zoom(50.f)));

		Node hexa = new Node(ellipse, pitch, null);

		// System.out.println(hexa.getDecoration() + " " +
		// hexa.getShape().getBounds2D());
		this.hexaList.add(hexa);
	}

	public void drawEdge(Graphics g, int X1, int Y1, int X2, int Y2) {
		float pi1 = _orig.x + X1 * _dir1.x + Y1 * _dir2.x;
		float pj1 = _orig.y + X1 * _dir1.y + Y1 * _dir2.y;
		float pi2 = _orig.x + X2 * _dir1.x + Y2 * _dir2.x;
		float pj2 = _orig.y + X2 * _dir1.y + Y2 * _dir2.y;
		if (g != null) {
			g.setColor(_m.get_edge_color());
			g.drawLine((int) pi1, (int) pj1, (int) pi2, (int) pj2);
		}
		Edge edge = new Edge(new Point2D.Double(pi1, pj1), new Point2D.Double(pi2, pj2), null);
		this.edgeList.add(edge);
	}

	private float _stroke = 2.f;
	private float _font_size = 18.f;
	private String _font_name = "Times New Roman";
	private int _font_style = Font.PLAIN;

	public void set_stroke(float stroke) {
		_stroke = stroke;
	}

	public void set_font_size(float psize) {
		_font_size = psize;
	}

	public void set_font_name(String pname) {
		_font_name = pname;
	}

	public void set_font_style(int pstyle) {
		_font_style = pstyle;
	}

	// TODO c'est ici que a se passe!!!!!
	public void paint(Graphics2D g) {

		g.setStroke(new BasicStroke(_p.get_zoom(_stroke)));
		g.setFont(new Font(_font_name, _font_style, (int) _p.get_zoom(_font_size)));

		Vector2 p = globalToGrid(0, 0);
		Vector2 e0 = _edges.get(0);

		if (_m.draw_edge()) {
			drawEdges(g, p, e0);
		}

		if (_m.draw_node()) {
			drawNodes(g, p, e0);
		}

	}

	private void drawNodes(Graphics2D g, Vector2 p, Vector2 e0) {
		int r;
		int radius;

		hexaList = new ArrayList<Node>();
		drawNode(g, (int) p.x, (int) p.y);
		for (radius = 0; radius <= _grid_radius; radius++) {
			float nx = p.x + radius * e0.x;
			float ny = p.y + radius * e0.y;

			Iterator<Vector2> dedge = _dedges.iterator();
			while (dedge.hasNext()) {
				Vector2 de = dedge.next();
				for (r = 0; r < radius; r++) {
					drawNode(g, (int) nx, (int) ny);
					nx += de.x;
					ny += de.y;
				}
			}
		}
	}

	private void drawEdges(Graphics2D g, Vector2 p, Vector2 e0) {
		int r;
		int radius;
		edgeList = new ArrayList<Edge>();
		// createTriangle(_edges);
		Iterator<Vector2> edge = _edges.iterator();
		while (edge.hasNext()) {
			Vector2 e = edge.next();
			drawEdge(g, (int) p.x, (int) p.y, (int) (p.x + e.x), (int) (p.y + e.y));
		}
		for (radius = 0; radius <= _grid_radius; radius++) {
			float nx = p.x + radius * e0.x;
			float ny = p.y + radius * e0.y;

			Iterator<Vector2> dedge = _dedges.iterator();
			while (dedge.hasNext()) {
				Vector2 de = dedge.next();
				for (r = 0; r < radius; r++) {
					edge = _edges.iterator();
					while (edge.hasNext()) {
						Vector2 e = edge.next();
						drawEdge(g, (int) nx, (int) ny, (int) (nx + e.x), (int) (ny + e.y));
					}
					nx += de.x;
					ny += de.y;
				}
			}
		}
	}

	protected void set_embedding(GridLayerEmbedding _embedding) {
		this._embedding = _embedding;
		this.on_transformation();
	}

	public void set_embedding(Subgrid sg) {
		set_embedding((GridLayerEmbedding) sg);
	}

	public ArrayList<Node> getHexaList(int width, int height, float zoom, Angle angle) {

		Vector2 p = globalToGrid(0, 0);
		Vector2 e0 = _edges.get(0);
		drawNodes(null, p, e0);

		Rectangle2D.Double bounds = new Rectangle2D.Double(0, 0, width, height);
		ArrayList<Node> goodNodes = new ArrayList<Node>();
		for (Node h : hexaList) {
			if (bounds.intersects(h.getShape().getBounds2D())) {
				goodNodes.add(h);

			}
		}

		return goodNodes;
	}

	public ArrayList<Edge> getEdgeList(int width, int height, float zoom, Angle angle) {

		Vector2 p = globalToGrid(0, 0);
		Vector2 e0 = _edges.get(0);

		drawEdges(null, p, e0);

		Rectangle2D.Double bounds = new Rectangle2D.Double(0, 0, width, height);
		ArrayList<Edge> goodEdges = new ArrayList<Edge>();
		for (Edge e : edgeList) {
			if (bounds.intersectsLine(new Line2D.Double(e.getP0(),e.getP1()))) {
				goodEdges.add(e);

			}
		}

		return goodEdges;
	}

}
