package Interface;

import Utils.Vector2;

public class Subgrid extends GridLayerEmbedding {

	private Vector2    _v1;
	private Vector2    _v2;
	private Vector2    _org;
	private GridLayer _l;

	private float _det, _detx, _dety;
	private boolean _col;
	
	public Subgrid(GridLayer l, int X0, int Y0, int X1, int Y1, int X2, int Y2) {
		_l  = l;
		_org = new Vector2(X0,Y0);
		_v1 = new Vector2(X1,Y1);
		_v2 = new Vector2(X2,Y2);
		update();
	}

	public Subgrid(GridLayer l, int X1, int Y1, int X2, int Y2) {
		this(l,0,0,X1,Y1,X2,Y2);
	}

	public Subgrid(GridLayer l) {
		this(l,0,0,1,0,0,1);
	}

	@Override
	public void gridToLocal(Vector2 p) {
		subgridToGrid(p);
		_l.gridToLocal(p);
	}

	public void subgridToGrid(Vector2 p) {
		float px = p.x * _v1.x + p.y * _v2.x;
		float py = p.x * _v1.y + p.y * _v2.y;
		p.x = px;
		p.y = py;
		p.translate(_org);
	}
	
	@Override
	public void localToGrid(Vector2 p) {
		_l.localToGrid(p);
		gridToSubgrid(p);
	}

	public void gridToSubgrid(Vector2 p) {
		p.untranslate(_org);
		float px, py;
		if (_col) {
			px = p.x * _detx + p.y * _dety;
			py = 0;
		} else {
			px = (_v2.x*p.y - _v2.y*p.x) / _det;
			py = (_v1.y*p.x - _v1.x*p.y) / _det;
		}
		p.x = Math.round(px);
		p.y = Math.round(py);
	}

	public void supergridToEuclidean(Vector2 p) {
		_l.gridToLocal(p);
	}
	
	@Override
	public float get_node_dist() {
		//TODO: change for a dependence with _l
		return 75.f;
	}

	public void set_org(int X, int Y) {
		_org = new Vector2(X,Y);
		update();
	}

	public void set_v1(int X, int Y) {
		assert((X!=0)||(Y!=0));
		_v1 = new Vector2(X,Y);
		update();
	}

	public void set_v2(int X, int Y) {
		assert((X!=0)||(Y!=0));
		_v2 = new Vector2(X,Y);
		update();
	}

	private void update() {
		_det = _v2.x*_v1.y - _v2.y*_v1.x;
		_col = (Math.round(_det)==0);
		if (_col) {
			_detx = (Math.round(_v1.x)==0)?0:(1/_v1.x);
			_dety = (Math.round(_v1.x)==0)?(1/_v1.y):0;
		}
	}
}
