package Main;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;

import javax.swing.JFrame;

import Interface.GridLayer;
import Interface.HexaGridLayer;
import Interface.JTransformPanel;
import Model.Music.PlanarUnfoldedTonnetz;
import Utils.Angle;
import Utils.JFramePicture;

public class GridFrame extends JFrame implements KeyListener {

	private static final long serialVersionUID = 1L;
	// Control & view
	private JTransformPanel _p;
	private HexaChord _h;
	private PlanarUnfoldedTonnetz _tonnetz;

	// Control & view
	private MusicInterviewer1 _m1;
	// private MusicInterviewer2 _m2;
	private boolean _draw_subgrid;
	private HexaGridLayer _l2;
	private HexaGridLayer _l3;

	private ArrayList<int[]> _current_model_chord_coords;

	public GridFrame(PlanarUnfoldedTonnetz t, HexaChord h) {

		super("Tonnetz : " + t.toString());
		_tonnetz = t;
		_h = h;
		_draw_subgrid = _h.get_draw_subgrid();
		_p = panel_factory();
		_p.add_layer(vertices_layer_factory()); // display vertices
		// _p.add_layer(edges_layer_factory()); // display edges
		pack();
		setVisible(false);
	}

	private GridLayer vertices_layer_factory() {
		// First layer: all notes, no edge
		_m1 = new MusicInterviewer1(_h, _tonnetz);
		GridLayer l1 = new HexaGridLayer(_p, _m1, 75.f);
		_l2 = (HexaGridLayer) l1;
		return l1;
	}

	// private HexaGridLayer edges_layer_factory() {
	// // Second layer: notes of the current chord tonnetz with edges
	// _m2 = new MusicInterviewer2(_h, _tonnetz, _m1);
	// if (_draw_subgrid)
	// _l2 = new HexaGridLayer(_p, _m2);
	// else
	// _l2 = new HexaGridLayer(_p, _m2, 75.f);
	//
	// return _l2;
	// }

	private JTransformPanel panel_factory() {
		addWindowListener(new CloseWindowProcessing(this, _h, _tonnetz));

		if (!(_h.getCurrentFrame()==null))
			setLocation(0, 600);
		setPreferredSize(new Dimension(400, 400));
		setResizable(true);
		// setFocusable(true);

		_p = new JTransformPanel(_h);
		// Building the GUI
		GridBagLayout layout = new GridBagLayout();
		setLayout(layout);

		GridBagConstraints c = new GridBagConstraints();

		// External padding (here, all margins are of 10 pixels)
		c.insets = new Insets(10, 10, 10, 10);

		// Insertion of a panel

		c.gridx = 0;
		c.gridy = 0;
		c.fill = GridBagConstraints.BOTH; // for the component to be resized in
											// vertical and horizontal direction
		c.weightx = 1.0; // for the component to take all the horizontal space
		c.weighty = 1.0; // for the component to take all the vertical space
		add(_p, c);

		// Adding listeners
		addKeyListener(this);
		return _p;
	}

	public void switch_draw_subgrid() {
		_draw_subgrid = !_draw_subgrid;
		System.out.println("switch " + _draw_subgrid);

		_l2.set_embedding(75.f);
		_l3.set_embedding(75.f);
	}

	public void repaint() {
		_p.repaint();
	}

	@Override
	public void keyPressed(KeyEvent evt) {
		_h.keyPressed(evt);
	}

	@Override
	public void keyReleased(KeyEvent evt) {
		_h.keyReleased(evt);
	}

	@Override
	public void keyTyped(KeyEvent evt) {
		_h.keyTyped(evt);
	}

	public JTransformPanel get_p() {
		return _p;
	}

	public PlanarUnfoldedTonnetz get_tonnetz() {
		return _tonnetz;
	}

	public void saveFrameAsJepg() {
		JFramePicture.save(this, _tonnetz.toString(), "png");
	}

	public ArrayList<int[]> get_current_model_chord_coords() {
		return _current_model_chord_coords;
	}

	// TODO Auto-generated
	// method stub
	public ArrayList<Node> getHexaList(int width, int height, float zoom, Angle angle) {
		// ArrayList<Hexa> hexaList = new ArrayList<Hexa>();
		// hexthis._m1.getHexaList();
		// hexaList.add(new Hexa(new Rectangle2D.Float(10, 10, 100, 100),
		// "teste"));
		// this.setBounds(this.getX(), this.getY(), width, height);
		// paint(this.getGraphics());
		this._p._zoom = zoom;
		this._p._rot = angle;
		this._p._w = width;
		this._p._w_2 = (float) width / 2;
		this._p._h = height;
		this._p._h_2 = (float) height / 2;
		this._l2.on_transformation();
		
		return this._l2.getHexaList(width, height, zoom, angle);
	}

	public ArrayList<Edge> getEdgeList(int width, int height, float zoom, Angle angle) {
		this._p._zoom = zoom;
		this._p._rot = angle;
		this._p._w = width;
		this._p._w_2 = (float) width / 2;
		this._p._h = height;
		this._p._h_2 = (float) height / 2;
		this._l2.on_transformation();
		
		return this._l2.getEdgeList(width, height, zoom, angle);
	}
}
