package Model.Music;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.TreeMap;
import java.util.TreeSet;


import Utils.Gcd;
import Utils.Table;


class TonnetzCoord extends ArrayList<Integer> implements Comparable<TonnetzCoord> {

	private static final long serialVersionUID = 1576471135047215860L;

	public TonnetzCoord(ArrayList<Integer> l) {
		super(l);
	}
	
	public TonnetzCoord() {
		super();
	}

	public TonnetzCoord(int[] l) {
		super();
		for (int i : l){
			this.add(i);
		}
	}

	@Override
	public int compareTo(TonnetzCoord l) {
		int cmp = ((Integer)(l.size())).compareTo(size());
		if (cmp != 0) return cmp;
		int i;
		for(i=0;i<size();i++) {
			cmp = get(i).compareTo(l.get(i));
			if (cmp != 0) return cmp;
		}
		return 0;
	}
	
	public TonnetzCoord plus(TonnetzCoord coord){
		if (this.size() != coord.size()) {
			System.err.println("Error : try to add different-sized coords");
			return null;
		}
		TonnetzCoord tc = new TonnetzCoord();
		for (int i=0;i<size();i++) tc.add(get(i)+coord.get(i));
		return tc;	
	}
	public void move_one_gen_more(int n) {
		set(n, this.get(n)+1);
		
	}
	public void move_one_gen_less(int n) {
		set(n, this.get(n)-1);
	}

}

class TonnetzGen extends TonnetzCoord {

	private static final long serialVersionUID = 2629006422639861313L;

	public TonnetzGen(ArrayList<Integer> l) {
		super(l);
		assert(l.size()!=0);
		Collections.sort(this);
		assert(this.get(0)>0);
	}
	
	public TonnetzGen(int[] l) {
		super(l);
		assert(l.length!=0);
		Collections.sort(this);
		assert(this.get(0)>0);		
	}

}

public abstract class PlanarUnfoldedTonnetz {

	
	protected int _N;
	protected Scale _scale;
	protected int _gcd;
	protected TonnetzGen _generators;
	protected TreeMap<Integer, TreeSet<TonnetzCoord>> _coords;
	protected HashSet<Integer> _pitch_class_set;	
	public int get_N() {
		return _N;
	}
		
	public int get_generator(int n) {
		return _generators.get(n);
	}

	public TonnetzGen get_generators() {
		return _generators;
	}
	
	public TonnetzCoord toCoord(int n) {
		if ((n%_gcd)!=0) return new TonnetzCoord(); // si la note n'existe pas dans le rseau (pas la bonne orbite)
		return toCoords(n).get(0);
	}

	
	private void build_coords() {
		int i;
		_coords = new TreeMap<Integer,TreeSet<TonnetzCoord>>();

		TonnetzCoord zero = new TonnetzCoord();
		//for(i=0;i<_dim;i++) zero.add(0);
		for(i=0;i<_generators.size();i++) zero.add(0);

		TreeSet<TonnetzCoord> workList = new TreeSet<TonnetzCoord>();
		workList.add(zero);
		_coords.put(0,workList);

		workList = new TreeSet<TonnetzCoord>();
		workList.add(zero);
		
		while(true) {
			TreeSet<TonnetzCoord> neoWorkList = new TreeSet<TonnetzCoord>();

			for(TonnetzCoord c : workList) {
				//for(i=0;i<_dim;i++) {
				for(i=0;i<_generators.size();i++) {
					int x = c.get(i);
					if (x>=0) {
						TonnetzCoord nc = new TonnetzCoord(c);
						nc.set(i, x+1);
						if (!_coords.containsKey(toNote(nc))) neoWorkList.add(nc);
					}
					if (x<=0) {
						TonnetzCoord nc = new TonnetzCoord(c);
						nc.set(i, x-1);
						if (!_coords.containsKey(toNote(nc))) neoWorkList.add(nc);
					}
				}
			}

			if (neoWorkList.size() == 0) break;

			for(TonnetzCoord c : neoWorkList) {
				int n = toNote(c);
				TreeSet<TonnetzCoord> l = _coords.get(n);
				if (l == null) {
					l = new TreeSet<TonnetzCoord>();
					_coords.put(n,l);
				}
				l.add(c);
			}
			
			workList=neoWorkList;
		}
		if (_N != Constant.N){
			TreeSet<TonnetzCoord> l;
			TreeMap<Integer,TreeSet<TonnetzCoord>> _coords2 = new TreeMap<Integer,TreeSet<TonnetzCoord>>();
			for (Integer k : _coords.keySet()){
				 l = _coords.get(k);
				_coords2.put(get_scale().get_pitch(k), l);
			}
			_coords=_coords2;
		}
	}

	public ArrayList<TonnetzCoord> toCoords(int n) {	// n = pitch class
		//assert(n>0);
		n = ((n%Constant.N) + Constant.N) % Constant.N;
		if ((n%_gcd)!=0) return new ArrayList<TonnetzCoord>();
		if (_coords==null) {
			build_coords();
		}
		return new ArrayList<TonnetzCoord>(_coords.get(n));
	}
	
	public int toNote(ArrayList<Integer> coord) {
		//assert(coord.size()==_dim);
		assert(coord.size()==_generators.size());
		int i;
		int n=0;
		//for(i=0;i<_dim;i++) n += coord.get(i)*_generators.get(i);
		for(i=0;i<_generators.size();i++) n += coord.get(i)*_generators.get(i);
		return Gcd.reduce(n,_N);
	}

	public static int[] to_2gen_coords(TonnetzCoord c){ // Ex Si dans [3,4,5] : [0,1,-1] => [-1,2] (puisque X = g0 et Y = -g2 dans les rseaux hexagonaux)
		int[] coord = new int[2];
		assert(c.size()==3) : "Error : method to_2gen_coords is only implemented for 3-generators simplicial Tonnetz for 2D-hexagonal representation";
		coord[0]=c.get(0)-c.get(1);
		coord[1]=-c.get(2)+c.get(1);
		return coord;
	}

	public static TonnetzCoord normalized_coord(TonnetzCoord c){
		if (c.size() != 3) System.err.println("norm Not yet implemented for non-hexagonal representations");
		TonnetzCoord coord = new TonnetzCoord();
		coord.add(c.get(0)-c.get(2));
		coord.add(c.get(1)-c.get(2));
		coord.add(0);
		return coord;
	}

	public static ArrayList<int[]> to_2gen_coords(ArrayList<TonnetzCoord> list){
		ArrayList<int[]> coord_list = new ArrayList<int[]>();
		for (TonnetzCoord c : list) coord_list.add(to_2gen_coords(c));
		return coord_list;
	}

	// TODO : peut tre optimis
	public static TonnetzCoord to_3gen_coords(int[] c){
		if (c.length!=2) System.err.println("Not yet implemented for more than 2-dimensional representations (i.e more than 2 coordinates");
		TonnetzCoord ton_coord = new TonnetzCoord();
//		ton_coord.add(c[0]);
//		ton_coord.add(0);
//		ton_coord.add(-c[1]);
		ton_coord.add(c[0]+c[1]);
		ton_coord.add(c[1]);
		ton_coord.add(0);
		return ton_coord;
	}

	public int get_orbitCount() {
		return _gcd;
	}

	public int getOrbit(int note) {
		note = Gcd.reduce(note, _N);
		return note%_gcd;
	}
	
	public int getOrbit(STChord ch) {
		int t[] = new int[_gcd];
		int i; for(i=0;i<_gcd;i++) t[i]=0;
		for(Integer n : ch) t[getOrbit(n)]++;
		int o=0;
		for(i=1;i<_gcd;i++) if (t[o]<t[i]) o=i;
		return o;
	}
	
	// renvoi les premires coord trouves associs au pitch n dans une List de TonnetzCoord. Si aucun lment ne correspond, null est renvoy
	public TonnetzCoord pitch_in_list(ArrayList<TonnetzCoord> list, int pitch){
		for (TonnetzCoord c : list) {
			System.out.println("coord : "+c+" ; coords to pitch : "+t_coords_to_pitch(c));
			if (t_coords_to_pitch(c)==pitch){
//				System.out.println("OUI");
				return c;
			} else {
//				System.out.println("NON : "+pitch+" "+t_coords_to_pitch(c));
			}			
		}
		return null;
	}
	
	// renvoi le pitch associ  des XYcoordonnes
	public int xy_coords_to_pitch(int[] coords){
		if (coords.length != 2) System.err.println("method coords_to_pitch implemented for XY coords only");
		return Gcd.reduce(coords[0] * get_generator(0) - coords[2] * get_generator(2), _N);		
	}

	// renvoi les pitchs (et leurs Tcoordonnees) voisins d'une position 
//	public Hashtable neighbor_pitchs(TonnetzCoord coord){
//		Hashtable table = new Hashtable();
//		TonnetzCoord tc;
//		for (int i=0;i<_generators.size();i++){
//			tc = new TonnetzCoord(coord);
//			tc.move_one_gen_more(i);
//			table.put(tc, t_coords_to_pitch(tc));
//			tc = new TonnetzCoord(coord);
//			tc.move_one_gen_less(i);
//			table.put(tc, t_coords_to_pitch(tc));
//			//table.put(coord.move_one_gen_more(i), t_coords_to_pitch(coord.move_one_gen_more(i)));
//			//table.put(coord.move_one_gen_less(i), t_coords_to_pitch(coord.move_one_gen_less(i)));
//		}
//		return table;
//	}
//	
//	// renvoi les coordonnes voisines d'une note
//	public ArrayList<TonnetzCoord> neighbor_coords(TonnetzCoord coord){
//		ArrayList<TonnetzCoord> list = new ArrayList<TonnetzCoord>();
//		TonnetzCoord tc;
//		for (int i=0;i<_generators.size();i++){
//			tc = new TonnetzCoord(coord);
//			tc.move_one_gen_more(i);
//			list.add(normalized_coord(tc));
//			tc = new TonnetzCoord(coord);
//			tc.move_one_gen_less(i);
//			list.add(normalized_coord(tc));
//		}
//		return list;
//	}
//
//	// renvoi la liste des coordonnes des plus proches pitchs d'une certaine position. Renvoi vide si le pitch n'est pas dans le rseau
//	public ArrayList<TonnetzCoord> closest_pitches_Tcoords(int pitch, TonnetzCoord coords){
//		ArrayList<TonnetzCoord> pitches_coords = new ArrayList<TonnetzCoord>();
//		Hashtable table = neighbor_pitchs(coords);
//		return pitches_coords;
//	}
//	
//	// renvoi le cercle largit avec UNIQUEMENT les coord dont le pitch associ n'appartenaient pas au cercle avant largissement
//	public ArrayList<TonnetzCoord> enlarge_circle(ArrayList<TonnetzCoord> prev_c){
//		if (prev_c.size()==1) return neighbor_coords(prev_c.get(0));
//		HashSet<TonnetzCoord> new_c = new HashSet<TonnetzCoord>();
//		for (TonnetzCoord c : prev_c) {
//			ArrayList<TonnetzCoord> loc = neighbor_coords(c);
//			for (TonnetzCoord cc : loc) if (!prev_c.contains(cc)) new_c.add(cc);
//		}
//		return new ArrayList<TonnetzCoord>(new_c);
//	}
//
//	public ArrayList<TonnetzCoord> first_chord_coords(STChord c){
//		ArrayList<TonnetzCoord> coords_set = new ArrayList<TonnetzCoord>();
//		STChord to_draw = new STChord();
//		for (int i : c){
//			if (_pitch_class_set.contains(i)) to_draw.add(i);
//		}
//		TonnetzCoord searched_note_coords = toCoord(to_draw.get_smallest_pitch());
//		coords_set.add(searched_note_coords);
//		if (to_draw.size() == 1) return coords_set;
//		to_draw.remove(to_draw.get_smallest_pitch());
//		ArrayList<TonnetzCoord> circle = coords_set;
//
//		int n=5;
//			while(to_draw.size()!=0 && n>0){
//				circle = enlarge_circle(circle);
//				int searched_pitch = to_draw.get_smallest_pitch();
//				searched_note_coords = pitch_in_list(circle, searched_pitch);
//				if (searched_note_coords!=null) {
//					coords_set.add(searched_note_coords);
//					to_draw.remove(searched_pitch);
//					circle = coords_set;
//				}
//			n--;
//			}
//		return coords_set;
//	}
//
//	public ArrayList<TonnetzCoord> coords_subset(STChord c, ArrayList<TonnetzCoord> coords_set){
//		ArrayList<TonnetzCoord> subset = new ArrayList<TonnetzCoord>();
//		for (TonnetzCoord coord : coords_set){
//			if (c.contains(t_coords_to_pitch(coord))){
//				subset.add(coord);
//			}
//		}
//		return subset;
//	}
//
//	public ArrayList<TonnetzCoord> n_chord_coords(STChord c, ArrayList<TonnetzCoord> prev_coords){
////		System.out.println("prev cord : "+prev_coords);
//		ArrayList<TonnetzCoord> coords_set = new ArrayList<TonnetzCoord>();
//		STChord to_draw = new STChord();
//		for (int i : c){
//			if (_pitch_class_set.contains(i)) to_draw.add(i);
//		}
//		
//		for (TonnetzCoord tc : prev_coords){
//			if (c.contains(t_coords_to_pitch(tc))) {
//				coords_set.add(tc);
//				to_draw.remove(t_coords_to_pitch(tc));
//			}
//		}
////		System.out.println("to_draw : "+to_draw);
//		// Premier cas : le nouvel accord est un sous-accord du prcdent -> pas de recherche
//		if (to_draw.isEmpty()) return coords_set;
//		
//		ArrayList<TonnetzCoord> circle = coords_set;
//
//		// Second cas : le nouvel accord n'a aucune note en commun avec le prcdent. Le cercle de recherche grossit  partir de l'accord prcdent
//		if (coords_set.isEmpty()){
//			circle = prev_coords;
//			while (to_draw.size()!=0){
////				System.out.println("bibiim");
//				circle = enlarge_circle(circle);
//				ArrayList<TonnetzCoord> candidats = coords_subset(to_draw,circle);
//				if (candidats.size() > 0){
//					coords_set.add(candidats.get(0));
//					to_draw.remove(t_coords_to_pitch(candidats.get(0)));
//					circle = coords_set;
//				}
//			}
//
//		} else {
//		// Troisime cas : les deux accords ont un sous-ensemble en commun OU l'ancien est un sous ensemble du nouveau. Le cercle de recherche grossit  partir de l'intersection des 2.
//			ArrayList<TonnetzCoord> erased_notes = new ArrayList<TonnetzCoord>();
//			for (TonnetzCoord tc : prev_coords){
//				if (!coords_set.contains(tc)) erased_notes.add(tc);
//			}
//			circle = coords_set;
//			while (to_draw.size()!=0){
//				circle = enlarge_circle(circle);
//				TonnetzCoord new_choosen_coord = null;
//				ArrayList<TonnetzCoord> candidats = coords_subset(to_draw,circle);
//				if (candidats.size() > 1) {
//					new_choosen_coord = candidats.get(0);
//					for (TonnetzCoord cand : candidats) {
//						if (enlarge_circle(erased_notes).contains(cand)) new_choosen_coord = cand;
//					}
//				} else {
//					if (candidats.size() == 1) new_choosen_coord = candidats.get(0);
//				}
//				if (new_choosen_coord != null) {
//					coords_set.add(new_choosen_coord);
//					to_draw.remove(t_coords_to_pitch(new_choosen_coord));
//					circle = coords_set;
//				}
//			}
//
//		}
//		return coords_set;
//	}
//
//	public ArrayList<int[]> first_chord_XYcoords(STChord c){
//		ArrayList<TonnetzCoord> pop = first_chord_coords(c);
//		return to_2gen_coords(pop);
//	}
//	
//	public ArrayList<int[]> n_chord_XYcoords(STChord c, ArrayList<int[]> prev_coords){
//		if (prev_coords == null) return first_chord_XYcoords(c);
//		ArrayList<TonnetzCoord> prev = new ArrayList<TonnetzCoord>();
//		for (int[] co : prev_coords) {
//			prev.add(to_3gen_coords(co));
////			System.out.println(Table.toString(co));
////			System.out.println(to_3gen_coords(co));
//		}
//		return to_2gen_coords(n_chord_coords(c, prev));
//		//return to_2gen_coords(enlarge_circle(prev));
//	}


	public String toString() {
		//StringBuffer s = new StringBuffer("T["+Interval.get_name(_generators.get(0)));
		StringBuffer s = new StringBuffer("T["+_generators.get(0));
		for (int i = 1;i < _generators.size(); i++) {
			//s.append(","+Interval.get_name(_generators.get(i)));
			s.append(","+_generators.get(i));
		}
		s.append("]");
		return s.toString();
	}
	
	public String toHTMLString() {
		StringBuffer s = new StringBuffer("<html>T<sub>["+_generators.get(0));
		for (int i = 1;i < _generators.size(); i++) {
			s.append(","+_generators.get(i));
		}
		s.append("]</sub></html>");
		return s.toString();
	}
	
	//abstract public ArrayList<Integer> toCoord(int pitch);
	//abstract public TonnetzCoord toCoord(int pitch);

	public HashSet<Integer> get_pitch_class_set() {
		
		return _pitch_class_set;
	}

	// Fonction retournant les PC voisins d'une PC dans le tonnetz
	public HashSet<Integer> get_neighbors_pitch_class(int n) {
		HashSet<Integer> neighbors = new HashSet<Integer>();
		for (Integer g : _generators) {
			neighbors.add((n+g)%_N);
			neighbors.add((_N+n-g)%_N);
		}
		return neighbors;
	}
	
	// Fonction retournant les PC les plus en contact avec un accord dans le tonnetz
	public HashSet<Integer> closer_pitch_class(STChord c) {
		HashSet<Integer> s = new HashSet<Integer>();
		int max_neighbors = 0;
		for (Integer n : _pitch_class_set) {
			if (!c.member(n)) {
				int neighbors_in_chord = c.get_intersection(get_neighbors_pitch_class(n)).size();
				if (neighbors_in_chord >= max_neighbors) {
					if (neighbors_in_chord > max_neighbors) {
						max_neighbors = neighbors_in_chord;
						s.clear();
					}
					s.add(n);
				}
			}
		}
		return s;
	}

	// Fonction retournant une STChordSeq dont certains STChod ont t "augument" d'une note
	// lorsque celle ci tait unique note la plus proche
//	public STChordSeq get_extra_STvoice(STChordSeq seq) {
//		STChordSeq new_seq = new STChordSeq();
//		for (STChord c : seq) {
//			if (closer_pitch_class(c).size() == 1) {
//				STChord new_chord = c;
//				new_chord.add((Integer) closer_pitch_class(c).toArray()[0]);
//				new_seq.add(new_chord);
//			}
//			else new_seq.add(c);
//		}
//		return new_seq;
//	}
//	
//	public ChordSeq get_extra_voice(ChordSeq seq) {
//		ChordSeq new_seq = new ChordSeq();
//		for (Chord chord : seq) {
//			if (closer_pitch_class(chord.to_STChord(_N)).size() == 1) {
//				Chord c = chord;
//				c.add_pitch_class((Integer) closer_pitch_class(c.to_STChord(_N)).toArray()[0], _N);
//				new_seq.add(c);
//			} else new_seq.add(chord);
//		}
//		return new_seq;
//	}
//	
//	public Chord get_extra_voice_chord(Chord chord) {
////		Chord c = chord;
////		if
//		return chord;
//	}
	
	public TonnetzGen get_semi_generators() {
		int demiN = (_N/2)-(_N%2);
		ArrayList<Integer> list = new ArrayList<Integer>();
		for (Integer i : _generators) list.add(-Math.abs((i)-demiN)+demiN);
		return new TonnetzGen(list);
	}
	
	public IntervallicVector get_IntervalVectorU() {
		int M = (_N/2)-(_N%2);
		int iv[] = new int[M];
		for (int i : iv) iv[i]=0;
		for (int g : _generators) iv[Interval.MI(g, _N)-1]++;
		return new IntervallicVector(iv);
	}
	
	public IntervallicVector get_IntervalVectorF() {
		int M = (_N/2)-(_N%2);
		int iv[] = new int[M];
		for (int i : iv) iv[i]=0;
		for (int g : _generators) iv[Interval.MI(g, _N)-1]=1;
		return new IntervallicVector(iv);
	}
	
	// renvoi pitch associ  des Tcoordonnes
	public int t_coords_to_pitch(TonnetzCoord c){
	
		assert (_generators.size() == c.size()) : "Error : Coordinates don't match with the Tonnetz" ;
		if (_generators.size() != c.size()) {
			System.err.println("Error : Coordinates don't match with the Tonnetz");
		}
		int pitch = 0;
		for (int i = 0;i < c.size();i++){
			pitch = (pitch+(c.get(i)*_generators.get(i)));
//			pitch = get_scale().get_pitch((pitch+(c.get(i)*_generators.get(i)))%_N);
//			System.out.println("pitch atteint : "+pitch);
		}
		pitch = get_scale().get_pitch(pitch);
		while (pitch < 0) pitch += 12;
		return pitch%12;
	}
	
	public int get_connected_components_count(){
		return Gcd.gcd(_generators);
	}
	
	public int get_directions_count(){
		return _generators.size();
	}
	
	public static ArrayList<STIntervallicStructure> emerging_IS(HashSet<Integer> intervals){
		
		ArrayList<STIntervallicStructure> list = new ArrayList<STIntervallicStructure>();
		
		
		
		return list;
	}

	public Scale get_scale(){
		return _scale;
	}
	
	public static String latex_name(Collection<Integer> list){
		return "$T_{"+list+"}$";
	}

	// return a pitch (not a pitch-class) 
	public int get_real_pitch_from_coords(int X, int Y) {
		
		if (_N==7){
			int diatonic_pitch = X * get_generator(0) + (Y * (7-get_generator(2)));
			//System.out.println("diatonic pitch : "+diatonic_pitch+" -- "+((diatonic_pitch/7)*12));
			if (diatonic_pitch>=0) return _scale.get_pitch(diatonic_pitch%7) +60+((diatonic_pitch/7)*12);
			else return  _scale.get_pitch(diatonic_pitch%7) +48+(((diatonic_pitch+1)/7)*12);
			
		} 
		return X * get_generator(0) + (Y * (_N-get_generator(2))) +60;
	}

}

